import { errorHandler } from '@gtpl/shared-utils/error-messages';
import { Body, Controller, Get, Param, Post, Query, Req, Res, UploadedFiles, UseInterceptors } from '@nestjs/common';
import { FilesInterceptor } from '@nestjs/platform-express';
import { ApiConsumes, ApiParam, ApiQuery, ApiTags } from '@nestjs/swagger';
import { extname } from 'path';
import { diskStorage } from 'multer';
import { AddsService } from './adds.service';
import { AddsResponseModel, AllAddsResponseModel } from './dto/adds-repose-module';
import { AddsDTO } from './dto/adds.dto';
@ApiTags('adds')
@Controller('adds')
export class AddsController {
    constructor (
        private AddsService: AddsService,
    ){}

    /**
     * create text/url post 
     * @param queryParam which returns api version
     * @param postDto which passes details to be saved
     * @param request which returns headers 
     * @returns postResponse which returns the updated data along with status (true/false), errortype, intlcode, internal message which provides message the calling function.   
     */
    @Post('/createAdd')
    @ApiQuery({name:'apiVersion'})    
    @UseInterceptors(
		FilesInterceptor('image', 5, {
			//fileFilter: imageFileFilter,
			storage: diskStorage({
				destination: './public/adds',
				filename: function (req, file, callback) {
					const name = file.originalname.split('.')[0];
					const fileExtName = extname(file.originalname);
					const randomName = Array(4)
						.fill(null)
						.map(() => Math.round(Math.random() * 16).toString(16))
						.join('');
					callback(null, `${name}-${randomName}${fileExtName}`);
				}
			}),			
		}),
    )
    @ApiConsumes('multipart/form-data')
    async createAdd(@Query() queryParam,@UploadedFiles() image,@Body() AddsDTO:AddsDTO,@Req() request:Request): Promise<AddsResponseModel> {
        try {
            const response = [];
            image.forEach(file => {
                const fileReponse = {
                    imagePath: `adds/adds/${file.filename}`,
                    redirectUrl:AddsDTO.redirectUrl,
                    location:AddsDTO.location,
                    isHome:AddsDTO.isHome,
                    createdUser:AddsDTO.createdUser,
                    versionFlag:AddsDTO.versionFlag
                };
                response.push(fileReponse);
            })
            return await this.AddsService.createAdds(queryParam, response,);
        } catch (error) {
            return errorHandler(AddsResponseModel, error);
        }
    }

    @Get('/getAllActiveAdds')
    @ApiQuery({name:'apiVersion'})
    async getAllActiveAdds(@Query() queryParams, @Req() request:Request):Promise<AllAddsResponseModel> {
        try {
            if (queryParams.apiVersion == 1){
                return await this.AddsService.getAllActiveAdds(request);
            } else {
                return await this.AddsService.getAllActiveAdds(request);
            }
        } catch (error) {
            return errorHandler(AllAddsResponseModel, error);
        }
    }

    @Get('adds/:imgpath')
    @ApiParam({name:'imgpath'})
    seeUploadedFile(@Param('imgpath') Image, @Res() res) {
		try {
			return res.sendFile(Image, { root: './public/adds' });
		} catch (error) {
			return error.message;
		}
	}

    @Get('deleteAdd/:Id')
    @ApiQuery({name:'apiVersion'})
    @ApiParam({name:'Id'})
    async deleteAdd(@Query() queryParams, @Param('Id') Id, @Req() request:Request):Promise<AddsResponseModel> {
        try {
            return await this.AddsService.deleteAdd(queryParams, Id,request);
        } catch (error) {
            return errorHandler(AddsResponseModel, error);
        }
    }
}
