import { ErrorResponse } from '@gtpl/shared-models/common-models';
import { Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { getConnection, Repository } from 'typeorm';
import { NotificationsEntity } from '../notifications/notifications.entity';
import { PostPointsEntity } from '../post/post-points.entity';
import { PostEntity } from '../post/post.entity';
import { sendFCMNotifications } from '../shared/mobileOTP';
import { CommmentsEntity } from './comments.entity';
import { AllCommentsResponseModel, CommentsResponseModel } from './dto/comments-response-module';
import { CommentsDTO, DeletDto, GetCommentsByPostId, PaginationDto, } from './dto/comments.dto';

@Injectable()
export class CommentsService {
    constructor(
        @InjectRepository(CommmentsEntity)
        private commentsRepository: Repository<CommmentsEntity>,
        @InjectRepository(PostPointsEntity)
        private postPointsRepo:Repository<PostPointsEntity>,
        @InjectRepository(PostEntity)
        private postRepo:Repository<PostEntity>,
        @InjectRepository(NotificationsEntity)
        private notifyRepo:Repository<NotificationsEntity>,      
    ) {}

    async sendNotify(text) {
        return await sendFCMNotifications(text)
        .then(() => {      
          console.log('notification sent successufully');
          return true;
        })
        .catch((err) => {
          console.log('error in sending notification ', err);
          return false;
        });
    }

    async createComment(queryParams, commentsDTO:CommentsDTO, request: Request): Promise<CommentsResponseModel> {
        const response = new CommentsResponseModel();
        try {
            if(queryParams.apiVersion == 1) {
                const saveComment = await this.commentsRepository.save(commentsDTO);
                const fndCmnt = await this.commentsRepository.findOne({
                    relations:['user','user.profile'],
                    where:{Id:saveComment.Id}
                })
                const PostData = await this.postRepo.findOne({
                    relations:['user'],
                    where:{Id:commentsDTO.postId}
                })
                if(PostData.text.length > 10) {
                    PostData.text = PostData.text.substring(0,10)+' ...';
                }
                if(saveComment) {
                    let msg = {
                        to : `${PostData.user.fcmToken}`,
                        collapse_key : "type_a",
                        notification : {
                            body : `${fndCmnt.user.profile.firstName} has Commented Your Post ${PostData.text}`,
                            title: "Post - Comment"
                        },
                        data : {
                            body : `${fndCmnt.user.profile.firstName} has Commented Your Post ${PostData.text}`,
                            title: "Post - Comment",
                            key_1 : "",
                            key_2 : ""
                        }
                    }
                    await this.notifyRepo.save({
                        title:'Post - Comment',
                        message:`${fndCmnt.user.profile.firstName} has Commented Your Post ${PostData.text}`,
                        typeId:1,
                        userId:PostData.user.Id,
                        targetId:commentsDTO.postId,
                        status:true
                    })
                    if(PostData.user.fcmToken) {                        
                        await this.sendNotify(msg);
                    }
                    const pointsData = {
                        postId:commentsDTO.postId,
                        userId:PostData.userId,
                        actionBy:commentsDTO.userId,
                        points:2,
                        activity:'Comment Post',
                        versionFlag: 1
                    }
                    await this.postPointsRepo.save(pointsData)
                    response.status = true;
                    response.data   = saveComment;
                    response.statusCode = 50000;
                    response.internalMessage = 'Comment posted successfully';
                    response.intlCode = 11100;
                } else { 
                    throw new ErrorResponse(11106, 'Comment not posted')
                }
            } else {
                const saveComment = await this.commentsRepository.save(commentsDTO);
                const fndCmnt = await this.commentsRepository.findOne({
                    relations:['user','user.profile'],
                    where:{Id:saveComment.Id}
                })
                const PostData = await this.postRepo.findOne({
                    relations:['user'],
                    where:{Id:commentsDTO.postId}
                })
                if(PostData.text.length > 10) {
                    PostData.text = PostData.text.substring(0,10)+' ...';
                }
                if(saveComment) {
                    let msg = {
                        to : `${PostData.user.fcmToken}`,
                        collapse_key : "type_a",
                        notification : {
                            body : `${fndCmnt.user.profile.firstName} has Commented Your Post ${PostData.text}`,
                            title: "Post - Comment"
                        },
                        data : {
                            body : `${fndCmnt.user.profile.firstName} has Commented Your Post ${PostData.text}`,
                            title: "Post - Comment",
                            key_1 : "",
                            key_2 : ""
                        }
                    }
                    await this.notifyRepo.save({
                        title:'Post - Comment',
                        message:`${fndCmnt.user.profile.firstName} has Commented Your Post ${PostData.text}`,
                        typeId:1,
                        userId:PostData.user.Id,
                        targetId:commentsDTO.postId,
                        status:true
                    })
                    if(PostData.user.fcmToken) {                        
                        await this.sendNotify(msg);
                    }
                    const pointsData = {
                        postId:commentsDTO.postId,
                        userId:PostData.userId,
                        actionBy:commentsDTO.userId,
                        points:2,
                        activity:'Comment Post',
                        versionFlag: 1
                    }
                    await this.postPointsRepo.save(pointsData)
                    response.status = true;
                    response.data   = saveComment;
                    response.statusCode = 50000;
                    response.internalMessage = 'Comment posted successfully';
                    response.intlCode = 11100;
                } else { 
                    throw new ErrorResponse(11106, 'Comment not posted')
                }
            }
            return response;
        } catch (error) {
            throw error;
        }
    }

    async getCommentsByPostId(pageDto:PaginationDto, postId,request:Request): Promise<AllCommentsResponseModel> {
        const response = new AllCommentsResponseModel();
        try {
            if(pageDto.apiVersion == 1) {
                if(!pageDto.skip || pageDto.skip == null) {
                    pageDto.skip = 0;
                }
                const cmntData = await this.commentsRepository.find({
                    relations:["user","user.profile"],
                    where: {postId:postId,  status: true},
                    order: { createdAt: 'DESC' }
                });
                if(cmntData) {
                    const finData: any[] = [];
                    cmntData.map((c) => {
                        finData.push({
                            Id: c.Id,
                            postId: c.postId,
                            userId: c.userId,
                            comment: c.comment,
                            createdAt: c.createdAt,
                            firstName:c.user.profile.firstName != null ? c.user.profile.firstName:'',
                            profilePic:c.user.profile.profilePic != null ? `${process.env.WORKINGURL}/${c.user.profile.profilePic}` : ''
                        })
                    })
                    response.status = true;
                    response.data = finData;
                    response.statusCode = 50000;
                    response.internalMessage = 'data retrieved successfully';
                    response.intlCode = 11108;
                } else {
                    throw new ErrorResponse(99998, 'Data not found');
                }
            } else {
                if(!pageDto.skip || pageDto.skip == null) {
                    pageDto.skip = 0;
                }
                const cmntData = await this.commentsRepository.find({
                    relations:["user","user.profile"],
                    where: {postId:postId,  status: true},
                    order: { createdAt: 'DESC' }
                });
                if(cmntData) {
                    const finData: any[] = [];
                    cmntData.map((c) => {
                        finData.push({
                            Id: c.Id,
                            postId: c.postId,
                            userId: c.userId,
                            comment: c.comment,
                            createdAt: c.createdAt,
                            firstName:c.user.profile.firstName != null ? c.user.profile.firstName:'',
                            profilePic:c.user.profile.profilePic != null ? `${process.env.WORKINGURL}/${c.user.profile.profilePic}` : ''
                        })
                    })
                    response.status = true;
                    response.data = finData;
                    response.statusCode = 50000;
                    response.internalMessage = 'data retrieved successfully';
                    response.intlCode = 11108;
                } else {
                    throw new ErrorResponse(99998, 'Data not found');
                }
            }
            return response;
        } catch (error) {
            throw error;
        }
    }

    async getPostCmtsCount(postId) {
        try {
            const cmtCount = await getConnection()
            .createQueryBuilder()
            .select("count(*) as cnt")
            .from(CommmentsEntity, "Comments")
            .where('post_id=:da', { da: postId })
            .orderBy('created_at','ASC')
            .getRawOne();
            if(cmtCount) {
                return cmtCount;
            } else { 
                return null; 
            }
        } catch (error) {
            throw error;
        }
    }

    async deleteComment(queryParams, DeletDto:DeletDto, request:Request):Promise<CommentsResponseModel> {
        const response = new CommentsResponseModel();
        try {
            if(queryParams.apiVersion == 1) {
                const saveComment = await this.commentsRepository.delete(DeletDto.Id);                
                if(saveComment) {
                    response.status = true;
                    response.statusCode = 50000;
                    response.internalMessage = 'Comment deleted successfully';
                    response.intlCode = 11100;
                } else { 
                    throw new ErrorResponse(11106, 'Comment not deleted')
                }
            } else {
                const saveComment = await this.commentsRepository.delete(DeletDto.Id);                
                if(saveComment) {
                    response.status = true;
                    response.statusCode = 50000;
                    response.internalMessage = 'Comment deleted successfully';
                    response.intlCode = 11100;
                } else { 
                    throw new ErrorResponse(11106, 'Comment not deleted')
                }
            }
            return response;
        } catch (error) {
            throw error;
        }
    }
    
}
