import {IsInt, IsEmail, IsString, IsDateString, IsNumber, IsDate,IsAlphanumeric, MaxLength, Matches, IsOptional, IsNotEmpty, ValidateNested, isNotEmpty } from 'class-validator';
import { ApiProperty, ApiPropertyOptional } from '@nestjs/swagger';

export class EventDTO {
  @ApiProperty()
  @ApiPropertyOptional()
  @IsOptional()
  Id?: number;
  @ApiProperty()
  @MaxLength(500, { message: "eventtittle Code allows maximum 500 characters" })
  eventTitle:string;
  @ApiProperty()
  @MaxLength(500, { message: "eventdesc Code allows maximum 500 characters" })
  eventDesc:string;
  @ApiProperty()
  @ApiPropertyOptional()
  villageId:number | null;
  @ApiProperty()
  @IsNotEmpty()
  categoryId:number;
  @ApiProperty()
  @IsNotEmpty({message:"eventstartdate should not be empty"})
  eventStartDate: Date;
  @ApiProperty()
  @IsNotEmpty({message:"eventstarttime should not be empty"})
  eventStartTime: string;
  @ApiProperty()
  @IsNotEmpty({message:"eventenddate should not be empty"})
  eventEndDate: Date;
  @ApiProperty()
  @IsNotEmpty({message:"eventenddate should not be empty"})
  eventEndTime: string;
  @ApiProperty({
    default:0
  })
  @ApiPropertyOptional()
  public: number;
  @ApiProperty()
  @ApiPropertyOptional()
  status: boolean;
  @ApiProperty()
  @IsNotEmpty()
  userId:number; 
  @ApiProperty()
  guestIds?:any;
  createdAt : Date;
  @ApiProperty()
  @ApiPropertyOptional()
  createdUser : number;
  updatedAt : Date;
  updatedUser : number;
  @ApiProperty({
    minimum:1,
    default:1
  })
  versionFlag : number;
  @ApiProperty({ type: 'array', items: { type: 'string', format: 'binary' } })
  @ApiPropertyOptional()
  attachments?: any[];
}

export class EventUpdateDTO {
  @ApiProperty()
  Id: number;
  @ApiProperty()
  @MaxLength(500, { message: "eventtittle Code allows maximum 500 characters" })
  eventTitle:string;
  @ApiProperty()
  @MaxLength(500, { message: "eventdesc Code allows maximum 500 characters" })
  eventDesc:string;
  @ApiProperty()
  @ApiPropertyOptional()
  villageId:number;
  @ApiProperty()
  @IsNotEmpty()
  categoryId:number;
  @ApiProperty()
  @IsNotEmpty({message:"eventstartdate should not be empty"})
  eventStartDate: Date;
  @ApiProperty()
  @IsNotEmpty({message:"eventstarttime should not be empty"})
  eventStartTime: string;
  @ApiProperty()
  @IsNotEmpty({message:"eventenddate should not be empty"})
  eventEndDate: Date;
  @ApiProperty()
  @IsNotEmpty({message:"eventenddate should not be empty"})
  eventEndTime: string;
  @ApiProperty({
    default:0
  })
  @ApiPropertyOptional()
  public: number;
  @ApiProperty()
  @ApiPropertyOptional()
  status: boolean;
  @ApiProperty()
  @IsNotEmpty()
  userId:number; 
  @ApiProperty()
  guestIds?:any;
  createdAt : Date;
  createdUser : number;
  updatedAt : Date;
  @ApiProperty()
  @ApiPropertyOptional()
  updatedUser : number;
  @ApiProperty({
    minimum:1,
    default:1
  })
  versionFlag : number;
  @ApiProperty({ type: 'array', items: { type: 'string', format: 'binary' } })
  @ApiPropertyOptional()
  attachments?: any[];
}

export class EventStatusDTO {
  @ApiProperty()
  @IsNotEmpty({message:"Id should not be empty"})
  @IsNumber()
  Id: number;
  @ApiProperty()
  status: boolean;
  updatedAt : Date;
  @ApiProperty()
  @IsOptional()
  updatedUser : number;
}

export class GetEventDTO {
  @ApiProperty()
  @IsNotEmpty({message:"Id should not be empty"})    
  Id:number;   
}

export class EventPagDto {
  @ApiProperty()
  apiVersion:number;
  @ApiProperty()
  skip:number;
}

export class EventRSVPDto {
  @ApiProperty()
  eventId:number;
  @ApiProperty()
  userId:number;
  @ApiProperty()
  rsvp:number;
}