import {IsInt, IsEmail, IsString, IsDateString, IsNumber, IsDate,IsAlphanumeric, MaxLength, Matches, IsOptional, IsNotEmpty, ValidateNested, isNotEmpty } from 'class-validator';
import { ApiProperty, ApiPropertyOptional } from '@nestjs/swagger';
import { GendersList } from '@gtpl/shared-models/gtpl';

export class ProfileDTO {
  @ApiProperty()
  @ApiPropertyOptional()
  Id?: number;
  @ApiProperty({
    description: '10 digits mobile number - type: string but input should be only numeric ',
  })
  @ApiPropertyOptional()
  mobileNo?:string;
  @ApiProperty({
    description: 'user first name - type: string',
  })
  @MaxLength(500, { message: "FirstName allows maximum 500 characters" })
  firstName:string;
  @ApiProperty({
    description: 'user last name - type: string',
  })
  @ApiPropertyOptional()
  lastName: string;
  @ApiProperty({ description: 'user gender - type:string' ,enum: Object.values(GendersList) })
  @IsNotEmpty() gender: GendersList;
  @ApiProperty({
    description: 'user date of birth - type: date format: yyyy-mm-dd',
  })
  @ApiPropertyOptional()
  dob: Date;
  @ApiProperty({
    description: 'user address - type: string',
  })
  @MaxLength(500, { message: "Address allows maximum 500 characters" })
  address: string;
  @ApiProperty()
  @ApiPropertyOptional()
  wardnoId:number;
  @ApiProperty()
  @ApiPropertyOptional()
  houseNumber:string;
  @ApiProperty()
  @ApiPropertyOptional()
  mandalTown:string;
  @ApiProperty({
    description: 'user  - type: integer from villages by search',
  })
  @ApiPropertyOptional()
  villageId?: number;
  @ApiProperty()
  @ApiPropertyOptional()
  hamletId?:number;
  @ApiProperty({
    description: 'user  - type: integer user role',
  })
  @ApiPropertyOptional()
  roleId?: number;
  @ApiProperty({
    description: 'user email - type: string',
  })
  @ApiPropertyOptional()
  email?: string;
  @ApiProperty({
    description: 'logged in user id - type: integer',
  })
  userId:number;
  @ApiProperty()
  @ApiPropertyOptional()
  BloodDonner:boolean;
  @ApiProperty()
  @ApiPropertyOptional()
  BGroupId:number;
  @ApiProperty()
  @ApiPropertyOptional()
  profilePic?:string;
  @ApiProperty({ type: 'array', items: { type: 'string', format: 'binary' } , description:'upload image type:file'})
  @ApiPropertyOptional()
  otherPicImg?:any[];
  @ApiProperty()
  @ApiPropertyOptional()
  membershipImg?:string;
  @ApiProperty({ type: 'array', items: { type: 'string', format: 'binary' } , description:'upload image type:file'})
  @ApiPropertyOptional()
  membershipPic?:any[];
  @ApiProperty()
  @ApiPropertyOptional()
  membershipId:string;
  @ApiProperty()
  status: boolean;
  createdAt : Date;
  updatedAt : Date;
  @ApiProperty({
    description: 'api version number - type: integer',
    minimum:1,
    default:1
  })
  versionFlag : number;
}

export class ProfilePicDTO {
  @ApiProperty({
    description:'user id = type:integer'
  })
  @IsNotEmpty({message:"userId should not be empty"})
  userId: number;
  @ApiProperty({ type: 'array', items: { type: 'string', format: 'binary' } , description:'upload image type:file'})
  profilePic:any[];
}

