import { ApiConsumes, ApiHeader, ApiOperation, ApiParam, ApiQuery, ApiTags } from '@nestjs/swagger';
import { Controller, Post, Body, Logger, Get, UseGuards,Req, Query, Param, Put, Delete, UseInterceptors, UploadedFiles, Res} from '@nestjs/common';
import {  InformationMessageError, NoRecordsFound, InternalServerError, ValidationMessageError,  errorHandler} from '@gtpl/shared-utils/error-messages';
import { ProfileService } from './profile.service';
import { ProfileDTO, ProfilePicDTO } from './dto/profile.dto';
import { getMyNetworkResponseModel, GetProfilePicResponseModel, GetProfileResponseModel } from './dto/profile-response-model';
import { FileFieldsInterceptor, FilesInterceptor } from '@nestjs/platform-express';
import { extname } from 'path';
import { diskStorage } from 'multer';
import { imageFileFilter } from '../shared/file-upload';

@ApiTags('Profile')
@Controller('profile')
export class ProfileController {
    constructor(
        private profileService: ProfileService
    ){} 

    
    //@UseGuards(JwtAuthGuard)  
    @ApiHeader({
        name:'Authorization',
        description:'Authorization token'
    })  
    @Post('/updateProfile')
    @ApiQuery({name:'apiVersion'})
    @ApiOperation({summary:'create or update user profile'})
    @UseInterceptors(	
            FileFieldsInterceptor([{
              name:'membershipPic', maxCount: 1,
            },{name:'otherPicImg', maxCount: 1},],
            {
                    fileFilter: imageFileFilter,
                    storage: diskStorage({
                        destination: './public/profilePic',
                        filename: function (req, file, callback) {
                            const name = file.originalname.split('.')[0];
                            const fileExtName = extname(file.originalname);
                            const randomName = Array(4)
                                .fill(null)
                                .map(() => Math.round(Math.random() * 16).toString(16))
                                .join('');
                            callback(null, `${name}-${randomName}${fileExtName}`);
                        }
                    }),			
                }),	
          )
    @ApiConsumes('multipart/form-data')
    async updateProfile(@Query() queryParams,@UploadedFiles() files,@UploadedFiles() membershipPic, @Body() profileDTO:ProfileDTO, @Req() request: Request): Promise<GetProfileResponseModel> {
        try {
            const response = [];
            const response1 = [];
            if(files.otherPicImg) {
              files.otherPicImg.forEach(file => {
                const fileReponse =  `profile/profilePic/${file.filename}`;
               response.push(fileReponse);
              })
            }
            if(files.membershipPic) {
              files.membershipPic.forEach(file => {
                const fileReponse =  `profile/profilePic/${file.filename}`;
               response1.push(fileReponse);
              })
            }      
            return await this.profileService.updateProfile(queryParams, response,response1,profileDTO, request);
        } catch (error) {
            return errorHandler(GetProfileResponseModel, error);
        }
    }

    @ApiHeader({
        name:'Authorization',
        description:'Authorization token'
    })
    @Post('/updateProfilePic')
    @ApiOperation({summary:'update user profile picture'})
    @ApiQuery({name:'apiVersion'})
    @UseInterceptors(
		FilesInterceptor('profilePic', 1, {
			fileFilter: imageFileFilter,
			storage: diskStorage({
				destination: './public/profilePic',
				filename: function (req, file, callback) {
					const name = file.originalname.split('.')[0];
					const fileExtName = extname(file.originalname);
					const randomName = Array(4)
						.fill(null)
						.map(() => Math.round(Math.random() * 16).toString(16))
						.join('');
					callback(null, `${name}-${randomName}${fileExtName}`);
				}
			}),			
		}),
    )
    @ApiConsumes('multipart/form-data')
    async updateProfilePic(@Query() queryParam,@UploadedFiles() profilePic,@Body() profilePicDto:ProfilePicDTO,@Req() request:Request): Promise<GetProfilePicResponseModel> {
        try {
            const response = [];
            profilePic.forEach(file => {
                const fileReponse = {
                    profilePic: `profile/profilePic/${file.filename}`,                               
                };
               response.push(fileReponse);
            })
            return await this.profileService.updateProfilePic(queryParam, response,profilePicDto.userId);            
        } catch (error) {
            return errorHandler(GetProfilePicResponseModel, error);
        }
    }

    @Get('/profilePic/:imgpath')
    @ApiOperation({summary:'user profile picture display'})
    @ApiParam({name:'imgpath'})
    async seeUploadedPrifilePic(@Param('imgpath') Image, @Res() res) {
		try {
			return res.sendFile(Image, { root: './public/profilePic' });
		} catch (error) {
			return error.message;
		}
    }
    
    //@UseGuards(JwtAuthGuard)
    @ApiHeader({
        name:'Authorization',
        description:'Authorization token'
    })
    @Get('/getProfileByUserId/:userId')
    @ApiOperation({summary:'get user profile by userId'})
    @ApiParam({name:'userId'})
    @ApiQuery({name:'apiVersion'})
    async getProfileByUserId(@Query() queryParams, @Param('userId') userId, @Req() request:Request):Promise<getMyNetworkResponseModel> {
        try {
            return await this.profileService.getProfileByUserId(queryParams, userId, request);
        } catch (error) {
            return errorHandler(getMyNetworkResponseModel, error);
        }
    }

    

}























