import { errorHandler } from '@gtpl/shared-utils/error-messages';
import { Body, Controller, Post, Query, Req } from '@nestjs/common';
import { ApiHeader, ApiQuery, ApiTags } from '@nestjs/swagger';
import { GetReportsByCmntIdDTO, GetReportsByPostIdDTO, GetReportsByUserIdDTO, ReportDTO } from './dto/report.dto';
import { AllReportsResponseModel, ReportsResponseModel } from './dto/reports-response-module';
import { ReportService } from './report.service';
@ApiTags('Report')
@Controller('report')
export class ReportController {

    constructor (
        private ReportService:ReportService
    ) {}
    @ApiHeader({
        name:'Authorization',
        description:'Authorization token'
    })
    @Post('/createReport')
    @ApiQuery({name:'apiVersion'})
    async createReport(@Query() queryParams, @Body() reportDto:ReportDTO, @Req() request: Request): Promise<ReportsResponseModel> {
        try {
            return await this.ReportService.createReport(queryParams, reportDto, request);
        } catch (error) {
            return errorHandler(ReportsResponseModel, error);
        }
    }
    @ApiHeader({
        name:'Authorization',
        description:'Authorization token'
    })
    @Post('/getReportsByPostId')
    @ApiQuery({name:'apiVersion'})
    async getReportsByPostId(@Query() queryParams,@Body() getReportsByPostIdDto:GetReportsByPostIdDTO,request: Request): Promise<AllReportsResponseModel> {
        try {
            return await this.ReportService.getReportsByPostId(queryParams,getReportsByPostIdDto,request);
        } catch (error) {
            return errorHandler(AllReportsResponseModel, error);
        }
    }
    @ApiHeader({
        name:'Authorization',
        description:'Authorization token'
    })
    @Post('/getReportsByCommentId')
    @ApiQuery({name:'apiVersion'})
    async getReportsByCommentId(@Query() queryParams,@Body() getReportsByCmntIdDto:GetReportsByCmntIdDTO,request: Request): Promise<AllReportsResponseModel> {
        try {
            return await this.ReportService.getReportsByCommentId(queryParams,getReportsByCmntIdDto,request);
        } catch (error) {
            return errorHandler(AllReportsResponseModel, error);
        }
    }
    @ApiHeader({
        name:'Authorization',
        description:'Authorization token'
    })
    @Post('/getReportsByUserId')
    @ApiQuery({name:'apiVersion'})
    async getReportsByUserId(@Query() queryParams,@Body() getReportsByUserIdDTO:GetReportsByUserIdDTO,request: Request): Promise<AllReportsResponseModel> {
        try {
            return await this.ReportService.getReportsByUserId(queryParams,getReportsByUserIdDTO,request);
        } catch (error) {
            return errorHandler(AllReportsResponseModel, error);
        }
    }
}
