import { ErrorResponse } from '@gtpl/shared-models/common-models';
import { Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';
import { ProfileEntity } from '../profile/profile.entity';
import { sendFCMNotifications, sendTextLocalSMS } from '../shared/mobileOTP';
import {  RepresentationDTO, RepresentStatusUpdateDto } from './dto/reprentation.dto';
import { AllRepresentationResponseModel, RepresentationResponseModel, SignalRepresentationResponseModel } from './representation-response-module';
import { RepresentationEntity } from './representation.entity';
import { RepresentationFilesEntity } from './representationfiles.entity';
import * as fs from 'fs';
import { ProfileService } from '../profile/profile.service';
import { VillageService } from '../village/village.service';
import { NotificationsEntity } from '../notifications/notifications.entity';


@Injectable()
export class RepresentationService {
    constructor (
        @InjectRepository(RepresentationEntity)
        private representationRepository: Repository<RepresentationEntity>, 
        @InjectRepository(RepresentationFilesEntity)
        private representationfilesRepository: Repository<RepresentationFilesEntity>, 
        @InjectRepository(NotificationsEntity)
        private notifyRepo:Repository<NotificationsEntity>,
        private profileService: ProfileService,
        private villageService:VillageService,
    ) {}    
    
    async sendSMS(mobileNo:string, text: string,id:string) {
        return await sendTextLocalSMS(mobileNo, text,id)
        .then(() => {      
          console.log('sms sent successufully');
          return true;
        })
        .catch((err) => {
          console.log('error in sending sms ', err);
          return false;
        });
    }

    async sendNotify(text) {
        return await sendFCMNotifications(text)
        .then(() => {      
          console.log('notification sent successufully');
          return true;
        })
        .catch((err) => {
          console.log('error in sending notification ', err);
          return false;
        });
    }
      
    async createrepresentation(queryParams, RepresentationDTO:RepresentationDTO) {
        try {
            RepresentationDTO.status = true;
            RepresentationDTO.createdUser=RepresentationDTO.userId;
            const user = await this.profileService.getUserByUId(RepresentationDTO.userId);
            user.firstName = user.firstName != null? user.firstName:'';
            if(queryParams.apiVersion == 1) {
                const savedPost = await this.representationRepository.save(RepresentationDTO);  
                if(savedPost) {
                    const sentSMS = await this.sendSMS(RepresentationDTO.mobileNo, `Your Representation is registered by ${user.firstName}, your Representation ID is ${savedPost.Id} Please download netR App to get updates at Androd users : https://rb.gy/osvnjn and IOS users : https://rb.gy/6yhhus - SRI VARAHA SOLUTIONS`,process.env.BlkSMS_REPRS_TEMPID);
                    let msg = {
                        to : `${user.user.fcmToken}`,
                        collapse_key : "type_a",
                        notification : {
                            body : `Your Representation Created with ${savedPost.Id}`,
                            title: "Create Representation"
                        },
                        data : {
                            body : `Your Representation Created  with ${savedPost.Id}`,
                            title: "Create Representation",
                            key_1 : "",
                            key_2 : ""
                        }
                    }
                    await this.notifyRepo.save({
                        title:'Create Representation',
                        message:`Your Representation Created with ${savedPost.Id}`,
                        typeId:4,
                        userId:RepresentationDTO.userId,
                        targetId:savedPost.Id,
                        status:true
                    })
                    if(user.user.fcmToken) {
                        const nt = await this.sendNotify(msg);
                    }
                    const newUsr = await this.profileService.getProfileByPhone(RepresentationDTO.mobileNo);
                    if(newUsr) {
                        let msg1 = {
                            to : `${newUsr.user.fcmToken}`,
                            collapse_key : "type_a",
                            notification : {
                                body : `Your Representation is registered by ${user.firstName}, your Representation ID is ${savedPost.Id}`,
                                title: "Create Representation"
                            },
                            data : {
                                body : `Your Representation is registered by ${user.firstName}, your Representation ID is ${savedPost.Id}`,
                                title: "Create Representation",
                                key_1 : "",
                                key_2 : ""
                            }
                        }
                        await this.notifyRepo.save({
                            title:'Create Representation',
                            message:`Your Representation is registered by ${user.firstName}, your Representation ID is ${savedPost.Id}`,
                            typeId:4,
                            userId:newUsr.userId,
                            targetId:savedPost.Id,
                            status:true
                        })
                        
                        if(newUsr.user.fcmToken) {
                            await this.sendNotify(msg1);
                        }
                    }
                    return savedPost;
                } else {
                    return null;
                }
            } else {
                const savedPost = await this.representationRepository.save(RepresentationDTO);  
                if(savedPost) {
                    const sentSMS = await this.sendSMS(RepresentationDTO.mobileNo, `Your Representation is registered by ${user.firstName}, your Representation ID is ${savedPost.Id} Please download netR App to get updates at Androd users : https://rb.gy/osvnjn and IOS users : https://rb.gy/6yhhus - SRI VARAHA SOLUTIONS`,process.env.BlkSMS_REPRS_TEMPID);
                    let msg = {
                        to : `${user.user.fcmToken}`,
                        collapse_key : "type_a",
                        notification : {
                            body : `Your Representation Created with ${savedPost.Id}`,
                            title: "Create Representation"
                        },
                        data : {
                            body : `Your Representation Created  with ${savedPost.Id}`,
                            title: "Create Representation",
                            key_1 : "",
                            key_2 : ""
                        }
                    }
                    if(user.user.fcmToken) {
                        await this.notifyRepo.save({
                            title:'Create Representation',
                            message:`Your Representation Created with ${savedPost.Id}`,
                            typeId:4,
                            userId:RepresentationDTO.userId,
                            targetId:savedPost.Id,
                            status:true
                        })
                        const nt = await this.sendNotify(msg);
                    }
                    const newUsr = await this.profileService.getProfileByPhone(RepresentationDTO.mobileNo);
                    if(newUsr) {
                        let msg1 = {
                            to : `${newUsr.user.fcmToken}`,
                            collapse_key : "type_a",
                            notification : {
                                body : `Your Representation is registered by ${user.firstName}, your Representation ID is ${savedPost.Id}`,
                                title: "Create Representation"
                            },
                            data : {
                                body : `Your Representation is registered by ${user.firstName}, your Representation ID is ${savedPost.Id}`,
                                title: "Create Representation",
                                key_1 : "",
                                key_2 : ""
                            }
                        }
                        await this.notifyRepo.save({
                            title:'Create Representation',
                            message:`Your Representation is registered by ${user.firstName}, your Representation ID is ${savedPost.Id}`,
                            typeId:4,
                            userId:newUsr.userId,
                            targetId:savedPost.Id,
                            status:true
                        })
                        
                        if(newUsr.user.fcmToken) {
                            await this.sendNotify(msg1);
                        }
                    }
                    return savedPost;
                } else {
                    return null;
                }
            }
        }
        catch (error) {
            throw error;
        
        }
    }

    async createrepresentationfiles(queryParams, filesData, repId): Promise<RepresentationResponseModel> {
        const response = new RepresentationResponseModel();
        try {
            if(queryParams.apiVersion == 1) {
                const fileUpload = await this.representationfilesRepository.save(filesData);
                const data = await this.representationRepository.findOne({
                    where:{Id:repId}
                })
                if(fileUpload) {
                    response.status = true;
                    response.data   = data;
                    response.statusCode = 50000;
                    response.internalMessage = 'Representation saved successfully';
                    response.intlCode = 11100;
                } else { 
                    throw new ErrorResponse(11106, 'Representation not saved')
                }
            } else {
                const fileUpload = await this.representationfilesRepository.save(filesData);
                const data = await this.representationRepository.findOne({
                    where:{Id:repId}
                })
                if(fileUpload) {
                    response.status = true;
                    response.data   = data;
                    response.statusCode = 50000;
                    response.internalMessage = 'Representation saved successfully';
                    response.intlCode = 11100;
                } else { 
                    throw new ErrorResponse(11106, 'Representation not saved')
                }
            }      
            return response;
        } catch (error) {
            throw error;
        }
    }

    async updateRepresentation(queryParams, filesData, RepresentationDTO:RepresentationDTO): Promise<RepresentationResponseModel> {
        const response = new RepresentationResponseModel();
        try {
            const oldData = await this.representationRepository.findOne({
                where:{Id:RepresentationDTO.Id}
            })
            if(RepresentationDTO.text) {
                RepresentationDTO.text = RepresentationDTO.text
            } else {
                RepresentationDTO.text = oldData.text
            }
            if(RepresentationDTO.mobileNo) {
                RepresentationDTO.mobileNo = RepresentationDTO.mobileNo
            } else {
                RepresentationDTO.mobileNo = oldData.mobileNo
            }
            if(RepresentationDTO.firstName) {
                RepresentationDTO.firstName = RepresentationDTO.firstName
            } else {
                RepresentationDTO.firstName = oldData.firstName
            }
            if(RepresentationDTO.lastName) {
                RepresentationDTO.lastName = RepresentationDTO.lastName
            } else {
                RepresentationDTO.lastName = oldData.lastName
            }
            if(RepresentationDTO.address) {
                RepresentationDTO.address = RepresentationDTO.address
            } else {
                RepresentationDTO.address = oldData.address
            }
            if(RepresentationDTO.villageId) {
                RepresentationDTO.villageId = RepresentationDTO.villageId
            } else {
                RepresentationDTO.villageId = oldData.villageId
            }
            if(RepresentationDTO.email) {
                RepresentationDTO.email = RepresentationDTO.email
            } else {
                RepresentationDTO.email = oldData.email
            }
            if(RepresentationDTO.versionFlag) {
                RepresentationDTO.versionFlag = RepresentationDTO.versionFlag
            } else {
                RepresentationDTO.versionFlag = oldData.versionFlag
            }
            RepresentationDTO.updatedUser = RepresentationDTO.userId;
            let images: any[] = [];
            let finaObj = {
                text:RepresentationDTO.text,
                mobileNo:RepresentationDTO.mobileNo,
                firstName:RepresentationDTO.firstName,
                lastName:RepresentationDTO.lastName,
                address:RepresentationDTO.address,
                villageId:RepresentationDTO.villageId,
                email:RepresentationDTO.email,
                userId:RepresentationDTO.userId,
                updatedUser:RepresentationDTO.updatedUser
            }
            if(queryParams.apiVersion == 1) {
                const updatedData = await this.representationRepository.update(RepresentationDTO.Id,finaObj);
                const getOldFiles = await this.representationfilesRepository.find({
                    where:{representationId:RepresentationDTO.Id}
                })
                if(getOldFiles.length > 0 && filesData.length > 0)  {
                    getOldFiles.map((f) => {
                        var fname = f.attachmentsUrl.split('/')[2];
                        fs.unlinkSync('./public/representationImages/'+fname);
                    })
                    await this.representationfilesRepository.delete({representationId:RepresentationDTO.Id});
                }
                const fileUpload = await this.representationfilesRepository.save(filesData);
                const data = await this.representationRepository.findOne({
                    where: {Id:RepresentationDTO.Id},
                });
                if(!data) {
                    throw new ErrorResponse(99908,'representation record update failed')
                }
                const user = await this.profileService.getUserByUId(data.userId);
                const village = await this.villageService.getVillageById(data.villageId);
                const postImages = await this.representationfilesRepository.find({
                    where:{representationId:data.Id}
                });
                if(postImages) {
                    postImages.map((i) => {
                        images.push(`${process.env.WORKINGURL}/${i.attachmentsUrl}`)
                    })
                }
                const final = {
                    Id: data.Id,
                    text:data.text,
                    firstName: data.firstName,
                    lastName: data.lastName,
                    mobileNo: data.mobileNo,
                    email: data.email,
                    address: data.address,
                    villageId:data.villageId,
                    villageName:village!=null?village.villageName:'',
                    createdAt:data.createdAt,
                    userId:data.userId,
                    userName: user.firstName,
                    profilePic: user.profilePic != null ? `${process.env.WORKINGURL}/${user.profilePic}` : '',
                    postImages:images,                        
                }
                response.status = true;
                response.statusCode = 50000;
                response.internalMessage = 'representation record updated';
                response.intlCode = 11108;
                response.data = final;
            } else  {
                const updatedData = await this.representationRepository.update(RepresentationDTO.Id,finaObj);
                const getOldFiles = await this.representationfilesRepository.find({
                    where:{representationId:RepresentationDTO.Id}
                })
                if(getOldFiles.length > 0 && filesData.length > 0)  {
                    getOldFiles.map((f) => {
                        var fname = f.attachmentsUrl.split('/')[2];
                        fs.unlinkSync('./public/representationImages/'+fname);
                    })
                    await this.representationfilesRepository.delete({representationId:RepresentationDTO.Id});
                }
                const fileUpload = await this.representationfilesRepository.save(filesData);
                const data = await this.representationRepository.findOne({
                    where: {Id:RepresentationDTO.Id},
                });
                if(!data) {
                    throw new ErrorResponse(99908,'representation record update failed')
                }
                const user = await this.profileService.getUserByUId(data.userId);
                const village = await this.villageService.getVillageById(data.villageId);
                const postImages = await this.representationfilesRepository.find({
                    where:{representationId:data.Id}
                });
                if(postImages) {
                    postImages.map((i) => {
                        images.push(`${process.env.WORKINGURL}/${i.attachmentsUrl}`)
                    })
                }
                const final = {
                    Id: data.Id,
                    text:data.text,
                    firstName: data.firstName,
                    lastName: data.lastName,
                    mobileNo: data.mobileNo,
                    email: data.email,
                    address: data.address,
                    villageId:data.villageId,
                    villageName:village!=null?village.villageName:'',
                    createdAt:data.createdAt,
                    userId:data.userId,
                    userName: user.firstName,
                    profilePic: user.profilePic != null ? `${process.env.WORKINGURL}/${user.profilePic}` : '',
                    postImages:images,                        
                }
                response.status = true;
                response.statusCode = 50000;
                response.internalMessage = 'representation record updated';
                response.intlCode = 11108;
                response.data = final;
            }     
            return response;
        } catch (error) {
            throw error;
        }
    }
	
    async getAllActiverepresentation(queryParams,userId,request:Request):Promise<AllRepresentationResponseModel> {
        const response = new AllRepresentationResponseModel();
        try {
            let images: any[] = [];
            const finalData: any[] = [];
            const user = await this.profileService.getUserByUId(userId);
            if(queryParams.apiVersion == 1) {
                const data = await this.representationRepository.find({
                    where: [{status: true,userId:userId},
                        {status:true,mobileNo:user.mobileNo}],
                    order: { createdAt: 'DESC' },
                    skip:queryParams.skip,
                    take: 10
                });
                if(!data) {
                    throw new ErrorResponse(99908,'Data not found')
                }
                const promises = data.map(async (post) => {
                    const user = await this.profileService.getUserByUId(post.userId);
                    const village = await this.villageService.getVillageById(post.villageId);
                    const postImages = await this.representationfilesRepository.find({
                        where:{representationId:post.Id}
                    });
                    return {
                        Id: post.Id,
                        text:post.text,
                        firstName: post.firstName,
                        lastName: post.lastName,
                        mobileNo: post.mobileNo,
                        email: post.email,
                        address: post.address,
                        villageId:post.villageId,
                        villageName:village!=null?village.villageName:'',
                        createdAt:post.createdAt,
                        userId:post.userId,
                        userName: user!=null?user.firstName:'',
                        profilePic:user!=null?user.profilePic:null,
                        postImages:postImages,
                    }   
                });   
                const da = await Promise.all(promises);
                da.map((f) => {  
                    if(f.postImages) {
                        f.postImages.map((i) => {
                            images.push(`${process.env.WORKINGURL}/${i.attachmentsUrl}`)
                        })
                    }                        
                    finalData.push({
                        Id: f.Id,
                        text:f.text,
                        firstName: f.firstName,
                        lastName: f.lastName,
                        mobileNo: f.mobileNo,
                        email: f.email,
                        address: f.address,
                        villageId:f.villageId,
                        villageName:f.villageName,
                        createdAt:f.createdAt,
                        userId:f.userId,
                        userName: f.firstName,
                        profilePic: f.profilePic != null ? `${process.env.WORKINGURL}/${f.profilePic}` : '',
                        postImages:images,                        
                    }) 
                    images=[];
                }) 
                response.status = true;
                response.data = finalData;
                response.statusCode = 50000;
                response.internalMessage = 'Data retrieved successfully';
                response.intlCode = 11108;              
            }  else{
                const data = await this.representationRepository.find({
                    where: [{status: true,userId:userId},
                        {status:true,mobileNo:user.mobileNo}],
                    order: { createdAt: 'DESC' },
                    skip:queryParams.skip,
                    take: 10
                });
                if(!data) {
                    throw new ErrorResponse(99908,'Data not found')
                }
                const promises = data.map(async (post) => {
                    const user = await this.profileService.getUserByUId(post.userId);
                    const village = await this.villageService.getVillageById(post.villageId);
                    const postImages = await this.representationfilesRepository.find({
                        where:{representationId:post.Id}
                    });
                    return {
                        Id: post.Id,
                        text:post.text,
                        firstName: post.firstName,
                        lastName: post.lastName,
                        mobileNo: post.mobileNo,
                        email: post.email,
                        address: post.address,
                        villageId:post.villageId,
                        villageName:village!=null?village.villageName:'',
                        createdAt:post.createdAt,
                        userId:post.userId,
                        userName: user!=null?user.firstName:'',
                        profilePic:user!=null?user.profilePic:null,
                        postImages:postImages,
                    }   
                });   
                const da = await Promise.all(promises);
                da.map((f) => {  
                    if(f.postImages) {
                        f.postImages.map((i) => {
                            images.push(`${process.env.WORKINGURL}/${i.attachmentsUrl}`)
                        })
                    }                        
                    finalData.push({
                        Id: f.Id,
                        text:f.text,
                        firstName: f.firstName,
                        lastName: f.lastName,
                        mobileNo: f.mobileNo,
                        email: f.email,
                        address: f.address,
                        villageId:f.villageId,
                        villageName:f.villageName,
                        createdAt:f.createdAt,
                        userId:f.userId,
                        userName: f.firstName,
                        profilePic: f.profilePic != null ? `${process.env.WORKINGURL}/${f.profilePic}` : '',
                        postImages:images,                        
                    }) 
                    images=[];
                }) 
                response.status = true;
                response.data = finalData;
                response.statusCode = 50000;
                response.internalMessage = 'Data retrieved successfully';
                response.intlCode = 11108;              
            }
            return response;
        } catch (error) {
            throw error;
        }
    }


    async getRepresentationById(queryParams,Id,request:Request):Promise<SignalRepresentationResponseModel> {
        const response = new SignalRepresentationResponseModel();
        try {
            let images: any[] = [];
            if(queryParams.apiVersion == 1) {
                const data = await this.representationRepository.findOne({
                    where: {Id:Id},
                });
                if(!data) {
                    throw new ErrorResponse(99908,'Data not found')
                }
                const user = await this.profileService.getUserByUId(data.userId);
                const village = await this.villageService.getVillageById(data.villageId);
                const postImages = await this.representationfilesRepository.find({
                    where:{representationId:data.Id}
                });
                if(postImages) {
                    postImages.map((i) => {
                        images.push(`${process.env.WORKINGURL}/${i.attachmentsUrl}`)
                    })
                }
                const final = {
                    Id: data.Id,
                    text:data.text,
                    firstName: data.firstName,
                    lastName: data.lastName,
                    mobileNo: data.mobileNo,
                    email: data.email,
                    address: data.address,
                    villageId:data.villageId,
                    villageName:village!=null?village.villageName:'',
                    createdAt:data.createdAt,
                    userId:data.userId,
                    userName: user.firstName,
                    profilePic: user.profilePic != null ? `${process.env.WORKINGURL}/${user.profilePic}` : '',
                    postImages:images,                        
                }
                response.status = true;
                response.data = final;
                response.statusCode = 50000;
                response.internalMessage = 'Data retrieved successfully';
                response.intlCode = 11108;
            } else {
                const data = await this.representationRepository.findOne({
                    where: {Id:Id},
                });
                if(!data) {
                    throw new ErrorResponse(99908,'Data not found')
                }
                const user = await this.profileService.getUserByUId(data.userId);
                const village = await this.villageService.getVillageById(data.villageId);
                const postImages = await this.representationfilesRepository.find({
                    where:{representationId:data.Id}
                });
                if(postImages) {
                    postImages.map((i) => {
                        images.push(`${process.env.WORKINGURL}/${i.attachmentsUrl}`)
                    })
                }
                const final = {
                    Id: data.Id,
                    text:data.text,
                    firstName: data.firstName,
                    lastName: data.lastName,
                    mobileNo: data.mobileNo,
                    email: data.email,
                    address: data.address,
                    villageId:data.villageId,
                    villageName:village!=null?village.villageName:'',
                    createdAt:data.createdAt,
                    userId:data.userId,
                    userName: user.firstName,
                    profilePic: user.profilePic != null ? `${process.env.WORKINGURL}/${user.profilePic}` : '',
                    postImages:images,                        
                }
                response.status = true;
                response.data = final;
                response.statusCode = 50000;
                response.internalMessage = 'Data retrieved successfully';
                response.intlCode = 11108;
            }  
            return response;
        } catch (error) {
            throw error;
        }
    }

    async updateRepresentStatus(queryParams,RepresentStatusUpdateDto:RepresentStatusUpdateDto, request:Request):Promise<RepresentationResponseModel> {
        const response = new RepresentationResponseModel();
        try {
            const updateRole = await this.representationRepository.update(RepresentStatusUpdateDto.Id, RepresentStatusUpdateDto);
            if(updateRole) {
                response.status = true;
                response.statusCode = 50000;
                response.internalMessage = 'Representation deleted successfully';
                response.intlCode = 11108;
            } else {
                throw new ErrorResponse(99998, 'Representation delete failed')
            }
            return response;
        } catch (error) {
            throw error;
        }
    }

}
