import {IsInt, IsEmail, IsString, IsDateString, IsNumber, IsDate,IsAlphanumeric, MaxLength, Matches, IsOptional, IsNotEmpty, ValidateNested, isNotEmpty } from 'class-validator';
import { ApiProperty } from '@nestjs/swagger';

export class RolesDTO {

  @ApiProperty()
  @IsOptional()
  Id?: number;
  @ApiProperty()
  @IsNotEmpty({message:"role name should not be empty"})
  @MaxLength(45, { message: "role name allows 45 characters only. " })
  @Matches(new RegExp("^[a-zA-Z0-9\\s]+$"),{ message:"role name should be only alpha numeric" })
  roleName:string;
  @ApiProperty()
  @IsNotEmpty({message:"level should not be empty"})
  @IsNumber()
  level:number;
  @ApiProperty()
  status: boolean;
  createdAt : Date;
  @ApiProperty()
  @IsOptional()
  createdUser : number;
  updatedAt : Date;
  @ApiProperty()
  @IsOptional()
  updatedUser : number;
  @ApiProperty()
  versionFlag : number;
}

export class RoleUpdateDto {
  @ApiProperty()
  Id: number;
  @ApiProperty()
  @IsNotEmpty({message:"role name should not be empty"})
  @MaxLength(45, { message: "role name allows 45 characters only. " })
  @Matches(new RegExp("^[a-zA-Z0-9\\s]+$"),{ message:"role name should be only alpha numeric" })
  roleName:string;
  @ApiProperty()
  @IsNotEmpty({message:"level should not be empty"})
  @IsNumber()
  level:number;
  updatedAt : Date;
  @ApiProperty()
  @IsOptional()
  updatedUser : number;
  @ApiProperty()
  versionFlag : number;
} 

export class RoleStatusUpdateDto {
  @ApiProperty()
  Id: number;
  @ApiProperty()
  status: boolean;
  updatedAt : Date;
  @ApiProperty()
  @IsOptional()
  updatedUser : number;
  @ApiProperty()
  versionFlag : number;
}
