import { ErrorResponse } from "@gtpl/shared-models/common-models";
import { errorHandler } from "@gtpl/shared-utils/error-messages";
import { Injectable } from "@nestjs/common";
import { InjectRepository } from "@nestjs/typeorm";
import { Repository } from "typeorm";
import { SettingsResponseModel } from "./dto/settings-response.model";
import { SettingsDto } from "./dto/settings.dto";
import { SettingsEntity } from "./settings.entity";

@Injectable()
export class SettingsService { 
    constructor (
        @InjectRepository(SettingsEntity)
        private readonly settingRepo:Repository<SettingsEntity>,
    ){}

    async createNew(icon,splash,sDto: SettingsDto, request:Request): Promise<SettingsResponseModel> {
        const response = new SettingsResponseModel();        
        try {
            const checkExe = await this.settingRepo.findOne();  
            if(checkExe) { 
                throw new ErrorResponse(99998, 'record alredy existed! ')
            }
            if(icon.length > 0) {
                var iconUrl = icon[0];
            } else {
                var iconUrl = null;
            }
            if(splash.length > 0) {
                var splashUrl = splash[0];
            } else {
                var splashUrl = null;
            }
            sDto.iconUploadUrl = iconUrl;
            sDto.uploadUrl = splashUrl;
            const saveData = await this.settingRepo.save(sDto);
            if(saveData) {
                response.status = true;
                response.data   = saveData;
                response.statusCode = 50000;
                response.internalMessage = 'saved successfully';
                response.intlCode = 11108;
            } else {
                throw new ErrorResponse(99998, 'save failed')
            }            
            return response;
        } catch (error) {
            return errorHandler(SettingsResponseModel, error);
        }
    }

    async updateOld(icon,splash,sDto: SettingsDto, request:Request): Promise<SettingsResponseModel> {
        const response = new SettingsResponseModel();        
        try {
            const checkExe = await this.settingRepo.findOne();  
            if(checkExe) { 
                if(icon.length > 0) {
                    sDto.iconUploadUrl = icon[0];
                } else {
                    sDto.iconUploadUrl = checkExe.iconUrl;
                }
                if(splash.length > 0) {
                    sDto.uploadUrl  = splash[0];
                } else {
                    sDto.uploadUrl  = checkExe.uploadUrl;
                }
                if(sDto.Id > 0) {
                    const updated = await this.settingRepo.update(sDto.Id, sDto);
                    response.status = true;
                    response.data   = updated;
                    response.statusCode = 50000;
                    response.internalMessage = 'updated successfully';
                    response.intlCode = 11108;
                } else {
                    sDto.Id =1;
                    const updated = await this.settingRepo.update(sDto.Id, sDto);
                    response.status = true;
                    response.data   = updated;
                    response.statusCode = 50000;
                    response.internalMessage = 'updated successfully';
                    response.intlCode = 11108;
                }
            } else {
                throw new ErrorResponse(99998, 'update failed')
            }            
            return response;
        } catch (error) {
            return errorHandler(SettingsResponseModel, error);
        }
    }

    async getSettings():Promise<SettingsResponseModel> {
        const response = new SettingsResponseModel();
        try {
            const data = await this.settingRepo.findOne();
            if(data) {
                if(data.iconUploadUrl != null) {
                    data.iconUploadUrl = `${process.env.WORKINGURL}/${data.iconUploadUrl}`;
                }
                if(data.uploadUrl !=null) {
                    data.uploadUrl = `${process.env.WORKINGURL}/${data.uploadUrl}`;
                }
                response.status = true;
                response.data   = data;
                response.statusCode = 50000;
                response.internalMessage = 'get settings info successfully';
                response.intlCode = 11108;
            } else { 
                throw new ErrorResponse(99998, 'data not found')
            }            
            return response;           
        } catch (error) {
            return errorHandler(SettingsResponseModel, error);
        }
    }
}