import {IsInt, IsEmail, IsString, IsDateString, IsNumber, IsDate,IsAlphanumeric, MaxLength, Matches, IsOptional, IsNotEmpty, ValidateNested } from 'class-validator';
import { ApiHideProperty, ApiProperty, ApiPropertyOptional } from '@nestjs/swagger';
import { GendersList } from '@gtpl/shared-models/gtpl';

export class CreateUserDto {  
  @ApiProperty()
  @ApiPropertyOptional()
  Id?: number;
  @ApiProperty({
    description: '10 digits mobile number - type: string but input should be only numeric ',
  })
  mobileNo:string;
  @ApiProperty()
  @ApiPropertyOptional()
  membershipImg?:string;
  @ApiProperty({ type: 'array', items: { type: 'string', format: 'binary' } , description:'upload image type:file'})
  @ApiPropertyOptional()
  membershipPic?:any[];
  @ApiProperty()
  @ApiPropertyOptional()
  membershipId:string;
  @ApiProperty()
  @ApiPropertyOptional()
  otherPic?:string;
  @ApiProperty({ type: 'array', items: { type: 'string', format: 'binary' } , description:'upload image type:file'})
  @ApiPropertyOptional()
  otherPicImg?:any[];
  @ApiProperty()
  @ApiPropertyOptional()
  @IsOptional()
  pin?:string;
  @ApiProperty({
    description: 'user role'
  })
  roleId?:number;
  @ApiProperty()
  @ApiPropertyOptional()
  @ApiHideProperty()
  hierarchyId?:number;
  @ApiProperty({
    description:'user first name'
  })
  firstName:string;
  @ApiProperty({
    description:'user last name'
  })
  @ApiPropertyOptional()
  lastName?: string;
  @ApiProperty()
  @ApiPropertyOptional()
  gender?: string;
  @ApiProperty({description:'user date of birth'})
  @ApiPropertyOptional()
  dob?: Date;
  @ApiProperty()
  @ApiPropertyOptional()
  address?: string;
  @ApiProperty({description:'user village from search'})
  @ApiPropertyOptional()
  villageId?: number;  
  @ApiProperty({description:'user email'})
  @ApiPropertyOptional()
  email?: string;
  @ApiProperty({description:'user created by'})
  @ApiPropertyOptional()
  @ApiHideProperty()
  userId?:number;
  @ApiProperty()
  @ApiPropertyOptional()
  BloodDonner:boolean;
  @ApiProperty()
  @ApiPropertyOptional()
  BGroupId:number;
  @ApiProperty({default:true})
  status: boolean;
  @ApiProperty()
  @ApiPropertyOptional()
  @ApiHideProperty()
  lastLogin?: Date;
  createdAt : Date;
  @ApiProperty()
  @ApiPropertyOptional()
  @IsOptional()
  createdUser? : number;
  updatedAt : Date;
  updatedUser? : number;
  @ApiProperty({minimum:1,default:1})
  versionFlag : number;
}

export class UserDto {
  @ApiProperty()
  Id?: number;
  @ApiProperty()
  @IsNotEmpty()
  mobileNo:string;
  @ApiProperty()
  @ApiPropertyOptional()
  membershipId:string;
  @ApiProperty()
  isPinSet:boolean;
  @ApiProperty()
  pin?:string;
  @ApiProperty()
  @IsNotEmpty({message:"Role should not be empty"})
  roleId:number;
  @ApiProperty()
  hierarchyId?:number;
  @ApiProperty()
  status: boolean;
  @ApiProperty()
  lastLogin?: Date;
  createdAt : Date;
  @ApiProperty()
  @IsOptional()
  createdUser : number;
  updatedAt : Date;
  @ApiProperty()
  @IsOptional()
  updatedUser : number;
  @ApiProperty()
  versionFlag : number;
  @ApiProperty()
  fcmToken:string;
}

export class UserBasicDto {
  @ApiProperty()
  userId:number;
  @ApiProperty({
    description: '10 digits mobile number - type: string but input should be only numeric ',
  })
  @IsNotEmpty()
  @MaxLength(10, { message: "Mobile number allows 10 characters only. " })
  @Matches(new RegExp("^[0-9\\s]+$"),{ message:"Mobile number should be only numeric. " })
  mobileNo:string;
  @ApiProperty()
  roleId:number;
}

export class UserRoleDto {
  @ApiProperty()
  userId:number;
  @ApiProperty()
  roleId:number;
  @ApiProperty()
  updatedUser:number;
}

export class UserIdDto {
  @ApiProperty()
  userId:number;
}

export class SignUpDto {
  @ApiProperty()
  @ApiPropertyOptional()
  membershipId:string;
  @ApiProperty({
    description:'user first name'
  })
  @MaxLength(500, { message: "FirstName allows maximum 500 characters" })
  @IsNotEmpty({message:"FirstName should not be empty"})
  firstName:string;
  @ApiProperty({
    description: 'user date of birth - type: date format: yyyy-mm-dd',
  })
  @ApiPropertyOptional()
  dob: Date;
  @ApiProperty({
    description: '10 digits mobile number - type: string but input should be only numeric ',
  })
  @IsNotEmpty()
  mobileNo:string;
  @ApiProperty({ description: 'user gender - type:string' ,enum: Object.values(GendersList) })
  @IsNotEmpty() gender: GendersList;
  @ApiProperty()
  @ApiPropertyOptional()
  wardnoId: number;
  @ApiProperty()
  @ApiPropertyOptional()
  houseNumber:string;
  @ApiProperty()
  @ApiPropertyOptional()
  mandalTown:string;
  @ApiProperty()
  @ApiPropertyOptional()
  villageId:number;
  @ApiProperty()
  @ApiPropertyOptional()
  hamletId:number;
  @ApiProperty()
  @ApiPropertyOptional()
  membershipImg?:string;
  @ApiProperty({ type: 'array', items: { type: 'string', format: 'binary' } , description:'upload image type:file'})
  @ApiPropertyOptional()
  membershipPic?:any[];
  
}

export class LoginUserDto { 
  @ApiProperty({
    description: '10 digits mobile number - type: string but input should be only numeric or emailid ',
  })
  @IsNotEmpty()
  /* @MaxLength(10, { message: "Mobile number allows 10 characters only. " })
  @Matches(new RegExp("^[0-9\\s]+$"),{ message:"Mobile no should be only numeric. " }) */
  mobileNo:string;
  @ApiProperty()
  @IsNotEmpty()
  pin:string;
  @ApiProperty()
  fcmToken:string;
}

export class SetPinDto {
  @ApiProperty({
    description: '10 digits mobile number - type: string but input should be only numeric ',
  })
  @IsNotEmpty()
  @MaxLength(10, { message: "Mobile number allows 10 characters only. " })
  @Matches(new RegExp("^[0-9\\s]+$"),{ message:"Mobile no should be only numeric. " })
  mobileNo:string;
  @ApiProperty()
  @IsNotEmpty({message:"pin should not be empty"})
  pin:string;  
}

export class OtpDto {  
  @ApiProperty({
    description: '10 digits mobile number - type: string but input should be only numeric ',
  })
  @IsNotEmpty()
  @MaxLength(10, { message: "Mobile number allows 10 characters only. " })
  @Matches(new RegExp("^[0-9\\s]+$"),{ message:"Mobile no should be only numeric. " })
  mobileNo:string; 
}

export class VerifyOtpDto {  
  @ApiProperty({
    description: '10 digits mobile number - type: string but input should be only numeric ',
  })
  @IsNotEmpty()
  @MaxLength(10, { message: "Mobile number allows 10 characters only. " })
  @Matches(new RegExp("^[0-9\\s]+$"),{ message:"Mobile no should be only numeric. " })
  mobileNo:string;
  @ApiProperty({
    description: '4 digits otp number - type:integer'
  })
  @IsNotEmpty({message:"OTP should not be empty"})
  otp:number; 
}

export class userSearchDto {
  @ApiProperty()
  @IsNotEmpty({message:"search not empty"})
  searchTerm:string;
}

export class bGroupSearchDto {
  @ApiProperty()
  apiVersion?:number;
  @ApiProperty()
  skip:number;
  @ApiProperty()
  @ApiPropertyOptional()
  BGroupId:number;
  @ApiProperty()
  @ApiPropertyOptional()
  villageId?:number;
}



