import { ErrorResponse } from '@gtpl/shared-models/common-models';
import { Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Raw, Repository } from 'typeorm';
import { AllWelfaretypeResponseModel, WelfaretypeResponseModel } from './dto/welfaretype-response-module';
import { WelfaretypeDTO, WelfaretypeStatusDTO } from './dto/welfaretype.dto';
import { WelfaretypeEntity } from './welfaretype.entity';

@Injectable()
export class WelfaretypeService {

    constructor(
        @InjectRepository(WelfaretypeEntity)
        private WelfaretypeRepository: Repository<WelfaretypeEntity>,
    ) {}

    async createWelafaretype(queryParams, WelfaretypeDTO:WelfaretypeDTO, request: Request): Promise<WelfaretypeResponseModel> {
        const response = new WelfaretypeResponseModel();
        try {
            const ActivityExists = await this.getEventcategoryWithoutRelations(WelfaretypeDTO.welfareType);
            if(ActivityExists) {
                throw new ErrorResponse(11104, 'welfaretype already exists');
            } else {
                if(queryParams.apiVersion == 1) {
                    WelfaretypeDTO.versionFlag = queryParams.apiVersion;
                    const savedActivity = await this.WelfaretypeRepository.save(WelfaretypeDTO);
                    if(savedActivity) {
                            response.status = true;
                            response.data   = savedActivity;
                            response.statusCode = 50000;
                            response.internalMessage = 'welfaretype created sucessfully';
                            response.intlCode = 11100;
                        }                       
                     else { 
                        throw new ErrorResponse(11106, 'welfaretype  not created')
                    }
                
                } else {
                    WelfaretypeDTO.versionFlag = queryParams.apiVersion;
                    const savedActivity = await this.WelfaretypeRepository.save(WelfaretypeDTO);
                    if(savedActivity) {
                            response.status = true;
                            response.data   = savedActivity;
                            response.statusCode = 50000;
                            response.internalMessage = 'welfaretype created sucessfully';
                            response.intlCode = 11100;
                        }                       
                     else { 
                        throw new ErrorResponse(11106, 'welfaretype  not created')
                    }
                }
            }
            return response;
        } catch (error) {
            throw error;
        }
    }

    async getEventcategoryWithoutRelations(welfareType: string): Promise<WelfaretypeEntity> {
        const activityResponse = await this.WelfaretypeRepository.findOne({
            where: {welfareType: Raw(alias => `UPPER(welfare_type) = '${welfareType.toLocaleUpperCase()}'`), status: true},
        });
        if(activityResponse) {
            return activityResponse;
        } else {
            return null;
        }
    }

    async getWelfaretype(queryParams, request:Request): Promise<AllWelfaretypeResponseModel> {
        const response = new AllWelfaretypeResponseModel();
        try {
            if(queryParams.apiVersion == 1) {
                const activeTypes = await this.WelfaretypeRepository.find({
                    where: {status: true},
                    order: { createdAt: 'DESC' }
                });
                if(activeTypes) {
                    response.status = true;
                    response.data = activeTypes;
                    response.statusCode = 50000;
                    response.internalMessage = 'Data retrieved successfully';
                    response.intlCode = 11108;
                } else {
                    throw new ErrorResponse(99998, 'Data not found');
                }
            } else {
                const activeTypes = await this.WelfaretypeRepository.find({
                    where: {status: true},
                    order: { createdAt: 'DESC' }
                });
                if(activeTypes) {
                    response.status = true;
                    response.data = activeTypes;
                    response.statusCode = 50000;
                    response.internalMessage = 'Data retrieved successfully';
                    response.intlCode = 11108;
                } else {
                    throw new ErrorResponse(99998, 'Data not found');
                }
            }
            return response;
        } catch (error) {
            throw error;
        }
    }

    async updateWelfaretype(queryParams, WelfaretypeDTO:WelfaretypeDTO, request:Request): Promise<WelfaretypeResponseModel> {
        const response = new WelfaretypeResponseModel();
        try {
            if(queryParams.apiVersion == 1) {
                const updateActivity = await this.WelfaretypeRepository.update(WelfaretypeDTO.Id,WelfaretypeDTO);
                const updatedData = await this.WelfaretypeRepository.findOne({
                    where:{Id:WelfaretypeDTO.Id}
                });
                if(updateActivity) {
                   
                    response.status = true;
                    response.data   = updatedData;
                    response.statusCode = 50000;
                    response.internalMessage = 'welfaretype updated successfully';
                    response.intlCode = 11108;
                } else {
                    throw new ErrorResponse(99998, 'welfaretype update failed')
                }
            } else {
                const updateActivity = await this.WelfaretypeRepository.update(WelfaretypeDTO.Id,WelfaretypeDTO);
                const updatedData = await this.WelfaretypeRepository.findOne({
                    where:{Id:WelfaretypeDTO.Id}
                });
                if(updateActivity) {
                   
                    response.status = true;
                    response.data   = updatedData;
                    response.statusCode = 50000;
                    response.internalMessage = 'welfaretype updated successfully';
                    response.intlCode = 11108;
                } else {
                    throw new ErrorResponse(99998, 'welfaretype update failed')
                }
            }
            return response;
        } catch (error) {
            throw error;
        }
    }

    async updatewelfaretypeStatus(queryParams, WelfaretypeStatusDTO:WelfaretypeStatusDTO, request:Request): Promise<WelfaretypeResponseModel> {
        const response = new WelfaretypeResponseModel();
        try {
            if(queryParams.apiVersion == 1) {
                const updateStatus = await this.WelfaretypeRepository.update(WelfaretypeStatusDTO.Id, WelfaretypeStatusDTO);
                const updatedData = await this.WelfaretypeRepository.findOne({
                    where:{Id:WelfaretypeStatusDTO.Id}
                });
                if(updateStatus) {
                    response.status = true;
                    response.data   = updatedData;
                    response.statusCode = 50000;
                    response.internalMessage = 'Welfaretype status updated successfully';
                    response.intlCode = 11108;
                } else {
                    throw new ErrorResponse(99998, 'Welfaretype status update failed')
                }
            } else {
                const updateStatus = await this.WelfaretypeRepository.update(WelfaretypeStatusDTO.Id, WelfaretypeStatusDTO);
                const updatedData = await this.WelfaretypeRepository.findOne({
                    where:{Id:WelfaretypeStatusDTO.Id}
                });
                if(updateStatus) {
                    response.status = true;
                    response.data   = updatedData;
                    response.statusCode = 50000;
                    response.internalMessage = 'Welfaretype status updated successfully';
                    response.intlCode = 11108;
                } else {
                    throw new ErrorResponse(99998, 'Welfaretype status update failed')
                }
            }
            return response;
        } catch (error) {
            throw error;
        }
    }



}
