import { errorHandler } from '@gtpl/shared-utils/error-messages';
import { Body, Controller, Get, Param, Post, Query, Req, Res, UploadedFiles, UseInterceptors } from '@nestjs/common';
import { ApiConsumes, ApiHeader, ApiParam, ApiQuery, ApiTags } from '@nestjs/swagger';
import { DeleteYoutubeDTO, PaginationDto, YoutubeDTO } from './dto/youtube.dto';
import { AllYTubeResponseModel, YTubeResponseModel } from './youtube-module';
import { YoutubeService } from './youtube.service';
@ApiTags('YoutubeLink')
@Controller('youtubelinks')
export class YoutubeController {
    constructor (
        private YtService: YoutubeService,
    ){}

    @ApiHeader({
        name:'Authorization',
        description:'Authorization token'
    })
    @Post('/createLinks') 
    @ApiQuery({name:'apiVersion'})   
    async createYoutubeLink(@Query() queryParam,@Body() ytDto:YoutubeDTO,@Req() request:Request): Promise<YTubeResponseModel> {
        try {
            return await this.YtService.createYoutubeLink(queryParam,ytDto,request);
        } catch (error) {
            return errorHandler(YTubeResponseModel, error);
        }
    }

    @ApiHeader({
        name:'Authorization',
        description:'Authorization token'
    })
    @Post('/updateLinks') 
    @ApiQuery({name:'apiVersion'})   
    async updateYoutubeLink(@Query() queryParam,@Body() ytDto:YoutubeDTO,@Req() request:Request): Promise<YTubeResponseModel> {
        try {
            return await this.YtService.updateYoutubeLink(queryParam, ytDto, request);
        } catch (error) {
            return errorHandler(YTubeResponseModel, error);
        }
    }

    @ApiHeader({
        name:'Authorization',
        description:'Authorization token'
    })
    @Get('/getAllLinks')
    @ApiQuery({name:'apiVersion'})
    @ApiQuery({name:'skip'})
    async getAllLinks(@Query() ptDto:PaginationDto, @Req() request:Request):Promise<AllYTubeResponseModel> {
        try {           
            return await this.YtService.getAllLinks(ptDto,request);
        } catch (error) {
            return errorHandler(AllYTubeResponseModel, error);
        }
    }

    @ApiHeader({
        name:'Authorization',
        description:'Authorization token'
    })
    @Get('/getLinkById/:Id')
    @ApiParam({name:'Id'})
    @ApiQuery({name:'apiVersion'})
    async getLinkById(@Query() queryParams, @Param('Id') Id,@Req() request:Request):Promise<YTubeResponseModel> {
        try {           
            return await this.YtService.getLinkById(queryParams,Id, request);
        } catch (error) {
            return errorHandler(YTubeResponseModel, error);
        }
    }
    
    @ApiHeader({
        name:'Authorization',
        description:'Authorization token'
    })
    @Post('/deleteLink')
    @ApiQuery({name:'apiVersion'})
    async deleteLink(@Query() queryParams, @Body() delDto: DeleteYoutubeDTO, @Req() request: Request): Promise<YTubeResponseModel> {
        try {   
            return await this.YtService.deleteLink(queryParams,delDto, request);
        } catch (error) {
            return errorHandler(YTubeResponseModel, error);
        }
    }  
    
    
}
