import Event from "./dto/event";
import Message from "./dto/message";

export interface EventBusService {
    /**
     * Initializes the transport's internals
     */
    initialize(): Promise<void>;

    /**
     * Starts listening for messages on the bus, call this to start receiving messages across the bus.
     * NOTE: This method should be called after all required topics have been subscribed to, 
     * transports like Kafka do not support dynamically subscribing to new topics after the broker has started
     */
    listen(): Promise<void>;

    /**
     * Subscribes to the requested topic
     * @param topicName the topic to subscribe to
     * @param eventListener will be called when a message is received for the relevant topic
     */
    subscribe(topicName: string, eventListener: EventListener): Promise<void>

    /**
     * Publishes a message to the given topic
     * @param topicName the topic to publish the message to
     * @param message the message to be published
     */
    publish(topicName: string, message: Message): Promise<void>

    destroy(): Promise<void>
}

export interface EventListener {
    (event: Event): Promise<void>;
}