import { EventBusLogger, LogEntry, logLevel } from "./event-bus-logger";

export class DefaultLogger implements EventBusLogger {
    private context = "EventBus"

    setContext(context: string): void {
        this.context = context;
    }

    logCreator(): (level: string) => (entry: LogEntry) => void {
        return (level: string) => {
            return (event: LogEntry) => {
                console.log(`[${this.context}] [${this.toLogLevel(parseInt(level))}]`, event);
            };
        }
    }

    log(message: any, context?: string | undefined) {
        if (context === undefined) {
            console.log(`[${this.context}] ${message}`);
        } else {
            console.log(`[${this.context}] ${message}`, [context]);
        }
    }

    error(message: any, trace?: string | undefined, context?: string | undefined) {
        let elements = [];
        if (context !== undefined) elements.push(context);
        if (trace !== undefined) elements.push(trace);

        if (elements.length > 0) {
            console.error(`[${this.context}] ${message}`, elements);
        } else {
            console.error(`[${this.context}] ${message}`);
        }
    }

    warn(message: any, context?: string | undefined) {
        if (context === undefined) {
            console.warn(`[${this.context}] ${message}`);
        } else {
            console.warn(`[${this.context}] ${message}`, [context]);
        }
    }

    toLogLevel(level: Number) {
        switch (level) {
            case logLevel.ERROR:
            case logLevel.NOTHING:
                return 'error'
            case logLevel.WARN:
                return 'warn'
            case logLevel.INFO:
                return 'info'
            case logLevel.DEBUG:
                return 'debug'
        }
    }
}
