import Event from "../dto/event";
import Message from "../dto/message";

export interface TransportProvider {
    /**
     * Initializes the connections for the transport provider
     */
    initialize(): Promise<void>;

    /**
     * Subscribes to the given topic
     * @param topicName the topic to subscribe to
     */
    subscribe(topicName: string): Promise<void>;

    /**
     * Publishes a message to the given topic
     * @param topicName the topic to publish the message to
     * @param message the message to publish
     */
    publish(topicName: string, message: Message): Promise<void>;

    /**
     * Assigns a handler to handle new messages received across the transport
     * @param messageHandler the strategy to handle new messages
     */
    onMessage(messageHandler: (payload: Event) => Promise<void>): Promise<void>;

    /**
     * Closes all connections used in the transport and prepares for destruction
     */
    destroy(): Promise<void>;
}